#数据常量
INDEX = 'index'
DATE = 'date'
SYMBOL = 'symbol'
PRE_CLOSE_PRICE = 'pre_close_price'
CLOSE_PRICE = 'close_price'
OPEN_PRICE = 'open_price'
HIGH_PRICE = 'high_price'
LOW_PRICE = 'low_price'
TURNOVER_RATE = 'turnover_rate'
VOLUME = 'volume'
AMOUNT = 'amount'
TOTAL_MKT_VAL = 'total_market_value'
CIRC_MKT_VAL = 'circulate_market_value'
MEAN_RETURNS = 'return'
WEIGHTS = 'weights'

# BACKTEST
DAILY_P_RETURN = 'daily_p_return'
DAILY_P_RETURN_TOTAL = 'daily_p_return_total'
AVG_PROFIT = 'avg_profit'
GEOMETRIC_MEAN = 'geometric_mean'
MIN_HOLD_PROFIT = 'min_hold_profit'
MAX_HOLD_PROFIT = 'max_hold_profit'
MIN_HOLD_PROFIT_INDEX = 'min_hold_profit_index'
MAX_HOLD_PROFIT_INDEX = 'max_hold_profit_index'