PORTF_FRAC = 'frac'
PORTF_FRAC_MAX = 'frac_max'

PORTF_MVO_MRG = 'mean_return_gt'
PORTF_MVO_MRM = 'mean_return_multiplier'
PORTF_MVO_RM = 'return_min'
PORTF_MVO_MIQP = 'portfolio_miqp'

PORTF_MAX_SHARP_RF = 'risk_free_rate'
PORTF_MAX_SHARP_SLSQP = 'SLSQP'